#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

SHORT_NAME = "short_name"
TITLE = "title"
PARAMETERS = "parameters"
NAME = "name"
FORMAT_TYPE = "format_type"
REQUIRED = "required"
DEFAULT_VALUE = "default_value"
LABEL = "label"
HELP_STRING = "help_string"
HELP_LINK = "help_link"
FORM_CLASS = "form_class"
FORM_STYLE = "form_style"
POSSIBLE_VALUES = "possible_values"
MODULAR_ALERTS = "modular_alerts"
PLACEHOLDER = "placeholder"
LINK_TEXT = "link_text"
LINK_URL = "link_url"
LINK_TIP = "link_tip"
LINK_TYPE = "link_type"
ICON_PATH = "icon_path"
ALERT_PROPS = "alert_props"
RESULTS = "results"
RESULT = "result"
RESULTS_FILE = "results_file"
STDIN_FIELDS = "stdin_fields"
LARGE_ICON = "largeIcon"
